/**
  This program is free software: you can redistribute it and / or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see < https : //www.gnu.org/licenses/>.
**/


/* Includes -------------------------------------------------- */
#include <Servo.h>
#include <avr/wdt.h>

/* Defines ------------------------------------------------------------------ */
#define button_pin      2
#define right_servo_pin 5
#define left_servo_pin  6
#define right_led       7
#define left_led        8
#define left_qti        A0
#define middle_qti      A1
#define right_qti       A2
#define min_pulse       1300
#define max_pulse       1700
#define standstill      1500
#define qti_threshold
#define DEBUG

/* Global variables ------------------------------------------ */
Servo g_left_wheel;
Servo g_right_wheel;

// the following variables are unsigned longs because the time, measured in
// milliseconds, will quickly become a bigger number than can be stored in an int.
unsigned long lastDebounceTime = 0;  // the last time the output pin was toggled
unsigned long debounceDelay = 50;    // the debounce time; increase if the output flickers
int buttonState;             // the current reading from the input pin
int lastButtonState = LOW;   // the previous reading from the input pin

/* Private functions ------------------------------------------------- */
byte readQti (byte qti) {                       // Function to read current position on map
  digitalWrite(qti, HIGH);                      // Send an infrared signal
  delayMicroseconds(100);                       // Wait for 100uS, very important!
  digitalWrite(qti, LOW);                       // Set the pin low again
  return analogRead(qti);                       // Return the analog result (between 0 to 1023)
}

/* Function from: https://www.arduino.cc/en/tutorial/debounce */
byte buttonRead() {

  int reading = digitalRead(button_pin);
  if (reading != lastButtonState) {
    lastDebounceTime = millis();
  }

  if ((millis() - lastDebounceTime) > debounceDelay) {
    if (reading != buttonState) {
      buttonState = reading;
      if (buttonState == HIGH) {
        return 1;
      }
    }
  }

  lastButtonState = reading;
  return 0;
}

void reboot() {
  wdt_disable();
  wdt_enable(WDTO_15MS);
  while (1) {}
}

/* Arduino functions ---------------------------------------------------------------- */
void setup() {
  /* Start serial monitor */
  Serial.begin(9600);

  pinMode(button_pin, INPUT);

  int dark_surface = 0;
  int light_surface = 0;
  byte i = 0;

  Serial.println("See programm annab QTI sensori tumeda ja heleda pinna väärtuse keskmise 10 mõõtmise põhjal.");
  Serial.println("Pane roboti kõik QTI sensorid tumedale pinnale ja vajuta nuppu.");

  while (buttonRead() != 1) {
  }
  Serial.println("Hakkan mõõtma tumeda pinna keskmist väärtust.");
  for (i = 0; i < 10; i++) {
    dark_surface += readQti(left_qti) + readQti(right_qti);
    delay(100);
  }

  dark_surface = dark_surface / (i * 2);
  Serial.print("Tumeda pinna keskmine väärtus on: ");
  Serial.println(dark_surface);

  delay(500);

  Serial.println("Pane roboti kõik QTI sensorid heledale pinnale ja vajuta nuppu.");
  while (buttonRead() != 1) {
  }


  Serial.println("Hakkan mõõtma heleda pinna keskmist väärtust.");
  for (i = 0; i < 10; i++) {
    light_surface += readQti(left_qti) + readQti(right_qti);
  }
  light_surface = light_surface / (i * 2 );
  Serial.print("Heleda pinna keskmine väärtus on: ");
  Serial.println(light_surface);
  Serial.print("Soovituslik ""qti_threshold"" väärtus on: ");
  Serial.println((light_surface + dark_surface) / 2);


  Serial.println("Uuesti mõõtmiseks vajuta nuppu.");
  
  while (1) {
    if (buttonRead() == 1) {
      reboot();
    }
  }

}

void loop() {
}
